Class: LDAP::Mod

Inherits:
Object
  • Object
show all
Includes:
Treequel::LDAPModExtensions
Defined in:
lib/treequel/monkeypatches.rb

Overview

Include Treequel-specific extensions as a mixin.

Instance Method Summary collapse

Methods included from Treequel::LDAPModExtensions

#==

Constructor Details

#initialize(op, attribute, vals = []) ⇒ Mod

Override the initializer to keep the attribute around while the object is alive to prevent the underlying C String pointer from going away.

See line 151 of mod.c.



72
73
74
75
# File 'lib/treequel/monkeypatches.rb', line 72

def initialize( op, attribute, vals=[] )
	@attribute = attribute
	_initialize_ext( op, attribute, vals )
end